@@ -71,21 +71,23 @@ import toxicblend.toxicblend_pb2 as toxicblend_pb2
 bl_info = {
     "name": "Toxicblend MeshTools",
     "author": "EADF",
-    "version": (0, 0, 13),
+    "version": (0, 0, 18),
     "blender": (2, 92, 0),
     "location": "View3D > Sidebar > Edit Tab / Edit Mode Context Menu",
     "warning": "Communicates with a gRPC server on localhost",
     "description": "Tools for handling lines and linestrings in edit mesh mode",
-    #"doc_url": "{BLENDER_MANUAL_URL}/addons/mesh/toxicblend_meshtools.html",
+    # "doc_url": "{BLENDER_MANUAL_URL}/addons/mesh/toxicblend_meshtools.html",
     "category": "Mesh",
 }
 
 SERVER_URL = 'localhost:50069'
 
+
 def check_toxicblend_version():
     if toxicblend.__version__ != '0.2.0':
         raise ToxicblendException("pip package toxicblend version 0.2.0 is required")
 
+
 class ToxicblendException(Exception):
     def __init__(self, message):
         self.message = str(message)
@@ -236,6 +238,8 @@ def handle_response(pb_message):
 def handle_received_object(active_object, pb_message, remove_doubles_threshold=None, set_origin_to_cursor=False):
     only_edges = False
     packed_faces = False
+    remove_doubles = ( remove_doubles_threshold is not None ) and remove_doubles_threshold > 0
+
     for option in pb_message.options:
         if option.key == "ERROR":
             raise ToxicblendException(str(option.value))
@@ -243,13 +247,17 @@ def handle_received_object(active_object, pb_message, remove_doubles_threshold=N
             only_edges = True
         if option.key == "PACKED_FACES" and option.value == "True":
             packed_faces = True
+        if option.key == "REMOVE_DOUBLES" and option.value == "True":
+            remove_doubles = True
+            if remove_doubles_threshold is None or remove_doubles_threshold <= 0:
+                remove_doubles_threshold = 0.0001
 
     if len(pb_message.models) > 0:
-       pb_model = pb_message.models[0]
+        pb_model = pb_message.models[0]
     elif len(pb_message.models32) > 0:
-       pb_model = pb_message.models32[0]
+        pb_model = pb_message.models32[0]
     else:
-       raise ToxicblendException("No return models found")
+        raise ToxicblendException("No return models found")
 
     if not pb_model is None:
         (vertices, edges, faces, matrix) = get_pydata(pb_model, only_edges, packed_faces)
@@ -280,17 +288,19 @@ def handle_received_object(active_object, pb_message, remove_doubles_threshold=N
             if matrix:
                 active_object.matrix_world = matrix
 
-            if remove_doubles_threshold and remove_doubles_threshold > 0:
+            if remove_doubles:
                 # sometimes 'mode_set' does not take right away  :/
                 # bpy.ops.object.editmode_toggle()
                 bpy.ops.object.mode_set(mode='EDIT')
                 bpy.ops.mesh.remove_doubles(threshold=remove_doubles_threshold)
                 bpy.ops.object.editmode_toggle()
                 bpy.ops.object.mode_set(mode='OBJECT')
+                bpy.ops.object.mode_set(mode='EDIT')
             if set_origin_to_cursor:
                 bpy.ops.object.origin_set(type='ORIGIN_CURSOR')
 
 
+# todo: use vertex.calc_edge_angle()
 def angle_between_edges(p0, p1, p2):
     """ angle between the two vectors defined as p0->p1 and p1->p2
     return value in degrees """
@@ -489,10 +499,6 @@ class ToxicBlend_SelectCollinearEdges(Operator):
 
         angle_criteria = math.degrees(self.angle)
 
-        # angle_between_edges(p0,p1,p2)
-        # make_edge_key(v0, v1):
-        # append_value(dict_obj, key, value):
-
         vertex_dict = dict()  # key by vertex.index to [edges]
         already_selected = set()  # key by edge.index
         work_queue = set()  # edges
@@ -703,7 +709,8 @@ class Toxicblend_2D_Outline(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -784,7 +791,8 @@ class Toxicblend_Voronoi(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -831,7 +839,8 @@ class Toxicblend_Knife_Intersect(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -931,7 +940,8 @@ class Toxicblend_Centerline(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -992,7 +1002,8 @@ class Toxicblend_Voronoi_Mesh(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -1019,7 +1030,7 @@ class Toxicblend_Voxel(Operator):
         description="Voxel tube radius as a percentage of the total AABB",
         default=1.0,
         min=0.01,
-        max=4.9999,
+        max=19.9999,
         precision=6,
         subtype='PERCENTAGE'
     )
@@ -1034,6 +1045,12 @@ class Toxicblend_Voxel(Operator):
         subtype='FACTOR'
     )
 
+    backend_variant_items = (#("_BB", "Building blocks", "use building blocks backend"),
+                             ("_SAFT", "Saft", "use saft backend"),
+                             ("_FSN", "Fast surface nets", "use fast-surface-nets backend")
+                             )
+    cmd_backend: bpy.props.EnumProperty(name="Backend", items=backend_variant_items, default="_FSN")
+
     @classmethod
     def poll(cls, context):
         ob = context.active_object
@@ -1055,7 +1072,7 @@ class Toxicblend_Voxel(Operator):
                            ('grpc.max_receive_message_length', 512 * 1024 * 1024)]
             with grpc.insecure_channel(SERVER_URL, options=channel_opt) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
-                command = toxicblend_pb2.Command(command='voxel')
+                command = toxicblend_pb2.Command(command='voxel' + str(self.cmd_backend).lower())
                 build_pb_model(active_object, active_mesh, command.models32.add())
 
                 opt = command.options.add()
@@ -1069,7 +1086,8 @@ class Toxicblend_Voxel(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -1129,7 +1147,8 @@ class Toxicblend_Simplify(Operator):
                 response = stub.execute(command)
                 handle_response(response)
                 if len(response.models) > 0 or len(response.models32) > 0:
-                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
+                    print("client received options: ", len(response.options), " models64:", len(response.models),
+                          " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -1288,7 +1307,7 @@ class TB_MeshToolsProps(PropertyGroup):
         description="Voxel tube radius as a percentage of the total AABB",
         default=1.0,
         min=0.01,
-        max=4.9999,
+        max=19.9999,
         precision=6,
         subtype='PERCENTAGE'
     )
@@ -1303,6 +1322,12 @@ class TB_MeshToolsProps(PropertyGroup):
         subtype='FACTOR'
     )
 
+    voxel_backend_variant_items = (#("_BB", "Building blocks", "use building blocks backend"),
+                              ("_SAFT", "Saft", "use saft backend"),
+                              ("_FSN", "Fast surface nets", "use fast-surface-nets backend")
+                             )
+    voxel_cmd_backend: bpy.props.EnumProperty(name="Backend", items=voxel_backend_variant_items, default="_FSN")
+
 
 # draw function for integration in menus
 def menu_func(self, context):
