@@ -48,22 +48,22 @@ from bpy_extras.object_utils import AddObjectHelper
 # See install_as_blender_addon.md details
 
 bl_info = {
-    'name': "Toxicblend - Add Lindenmayer systems",
+    'name': "Toxicblend - Add parametric objects",
     "author": "EADF",
-    "version": (0, 0, 13),
+    "version": (0, 0, 18),
     "blender": (2, 92, 0),
-    # "location": "View3D > Sidebar > Edit Tab / Edit Mode Context Menu",
     "warning": "Communicates with a gRPC server on localhost",
-    'description': 'Generates a parametric Lindenmayer systems graph.',
-    # "doc_url": "{BLENDER_MANUAL_URL}/addons/mesh/toxicblend_lsystems.html",
-    "category": "Mesh",
+    'description': 'Generates a parametric objects.',
+    "category": "Object",
 }
 SERVER_URL = 'localhost:50069'
 
+
 def check_toxicblend_version():
     if toxicblend.__version__ != '0.2.0':
         raise ToxicblendException("pip package toxicblend version 0.2.0 is required")
 
+
 class ToxicblendException(Exception):
     def __init__(self, message):
         self.message = str(message)
@@ -98,7 +98,7 @@ def settings_write(self):
 def terminate():
     # update editmesh cached data
     obj = bpy.context.active_object
-    if (not obj is None ) and obj.mode == 'EDIT':
+    if (not obj is None) and obj.mode == 'EDIT':
         bmesh.update_edit_mesh(obj.data, loop_triangles=True, destructive=True)
 
 
@@ -142,8 +142,8 @@ def get_pydata(pb_model, only_edges=False, packed_faces=False):
         mat[2][0], mat[2][1], mat[2][2], mat[2][3] = pbm.m20, pbm.m21, pbm.m22, pbm.m23
         mat[3][0], mat[3][1], mat[3][2], mat[3][3] = pbm.m30, pbm.m31, pbm.m32, pbm.m33
 
-    #print("rv_vertices:", rv_vertices)
-    #print("rv_edges:", rv_edges)
+    # print("rv_vertices:", rv_vertices)
+    # print("rv_edges:", rv_edges)
 
     return rv_vertices, rv_edges, rv_faces, mat
 
@@ -166,7 +166,7 @@ def handle_received_object(dest_mesh, pb_message, remove_doubles_threshold=None,
             packed_faces = True
 
     if len(pb_message.models) > 0:
-         pb_model = pb_message.models[0]
+        pb_model = pb_message.models[0]
     elif len(pb_message.models32) > 0:
         pb_model = pb_message.models32[0]
     else:
@@ -174,7 +174,7 @@ def handle_received_object(dest_mesh, pb_message, remove_doubles_threshold=None,
     if not pb_model is None:
         (vertices, edges, faces, matrix) = get_pydata(pb_model, only_edges, packed_faces)
         if len(faces) > 0 or len(edges) > 0:
-            #new_mesh = bpy.data.meshes.new(pb_model.name)
+            # new_mesh = bpy.data.meshes.new(pb_model.name)
             # ob = bpy.data.objects.new(pb_model.name, new_mesh)
             # active_object.active = ob
             # ob.select = True
@@ -185,15 +185,15 @@ def handle_received_object(dest_mesh, pb_message, remove_doubles_threshold=None,
             dest_mesh.update(calc_edges=True)
             bm = bmesh.new()
             bm.from_mesh(dest_mesh)
-            #bpy.ops.object.mode_set(mode='OBJECT')
+            # bpy.ops.object.mode_set(mode='OBJECT')
             bm.to_mesh(dest_mesh)
-            #bpy.ops.object.mode_set(mode='EDIT')
+            # bpy.ops.object.mode_set(mode='EDIT')
 
-            #bm = bmesh.new()
-            #bm.from_mesh(new_mesh)
-            #bpy.ops.object.mode_set(mode='OBJECT')
-            #dest_mesh.to_mesh(dest_mesh)
-            #bpy.ops.object.mode_set(mode='EDIT')
+            # bm = bmesh.new()
+            # bm.from_mesh(new_mesh)
+            # bpy.ops.object.mode_set(mode='OBJECT')
+            # dest_mesh.to_mesh(dest_mesh)
+            # bpy.ops.object.mode_set(mode='EDIT')
 
 
 class TbAddLindenmayerSystems(bpy.types.Operator):
@@ -202,14 +202,14 @@ class TbAddLindenmayerSystems(bpy.types.Operator):
     bl_label = "Toxicblend:Add Lindenmayer systems"
     bl_options = {'REGISTER', 'UNDO'}  # enable undo for the operator.
 
-    iterations : bpy.props.IntProperty(name="Iterations", default=4, min=1, max=15)
+    iterations: bpy.props.IntProperty(name="Iterations", default=4, min=1, max=15)
 
-    location : bpy.props.FloatVectorProperty(
+    location: bpy.props.FloatVectorProperty(
         name="Location",
         subtype='TRANSLATION',
     )
 
-    rotation : bpy.props.FloatVectorProperty(
+    rotation: bpy.props.FloatVectorProperty(
         name="Rotation",
         subtype='EULER',
     )
@@ -221,7 +221,7 @@ class TbAddLindenmayerSystems(bpy.types.Operator):
         ('CURSOR', "3D Cursor", "Use the 3D cursor orientation for the new object")
     )
 
-    align : bpy.props.EnumProperty(
+    align: bpy.props.EnumProperty(
         name="Align",
         items=align_items,
         default='WORLD',
@@ -238,37 +238,38 @@ class TbAddLindenmayerSystems(bpy.types.Operator):
                          ("KOCH_CURVE", "Koch curve", "draw a Koch curve (keep iterations low)"),
                          ("KOCH_CURVE_3D", "Koch curve 3d", "draw a 3d Koch curve (keep iterations low)"),
                          ("KOCH_CURVE_ISLAND", "Koch curve island", "draw a quadratic Koch curve island"),
-                         ("KOCH_CURVE_ISLAND_3D", "Koch curve island 3d", "draw a 3d variant of quadratic Koch curve island"),
+                         ("KOCH_CURVE_ISLAND_3D", "Koch curve island 3d",
+                          "draw a 3d variant of quadratic Koch curve island"),
                          ("HILBERT_CURVE_3D", "Hilbert curve 3d", "draw Hilbert curve 3d (keep iterations very low)"),
-                         ("SIERPINSKI_GASKET","Sierpinski gasket", "draw Sierpinski gasket"),
-                         ("SIERPINSKI_GASKET_3D","Sierpinski gasket 3d", "draw a 3f Sierpinski gasket"),
-                         ("SIERPINSKI_TRIANGLE","Sierpinski triangle", "draw Sierpinski triangle"),
-                         ("GOSPER_CURVE","Gosper curve", "draw Gosper curve (keep iterations low)"),
-                         ("GOSPER_CURVE_3D","Gosper curve 3d", "draw a 3d Gosper curve (keep iterations low)"),
-                         ("CUSTOM_TURTLE","Custom turtle", "draw using custom turtle")
-                        )
-    cmd_variant : bpy.props.EnumProperty(name="Variant", items=cmd_variant_items, default="DRAGON_CURVE")
+                         ("SIERPINSKI_GASKET", "Sierpinski gasket", "draw Sierpinski gasket"),
+                         ("SIERPINSKI_GASKET_3D", "Sierpinski gasket 3d", "draw a 3f Sierpinski gasket"),
+                         ("SIERPINSKI_TRIANGLE", "Sierpinski triangle", "draw Sierpinski triangle"),
+                         ("GOSPER_CURVE", "Gosper curve", "draw Gosper curve (keep iterations low)"),
+                         ("GOSPER_CURVE_3D", "Gosper curve 3d", "draw a 3d Gosper curve (keep iterations low)"),
+                         ("CUSTOM_TURTLE", "Custom turtle", "draw using custom turtle")
+                         )
+    cmd_variant: bpy.props.EnumProperty(name="Variant", items=cmd_variant_items, default="DRAGON_CURVE")
 
-    custom_turtle0 : bpy.props.StringProperty(name="custom turtle 0", default='token("X", Turtle::Nop)?')
-    custom_turtle1 : bpy.props.StringProperty(name="custom turtle 1", default='token("Y", Turtle::Nop)?')
-    custom_turtle2 : bpy.props.StringProperty(name="custom turtle 2", default='token("F", Turtle::Forward(1))?')
-    custom_turtle3 : bpy.props.StringProperty(name="custom turtle 3", default='token("+", Turtle::Yaw(-90))?')
-    custom_turtle4 : bpy.props.StringProperty(name="custom turtle 4", default='token("-", Turtle::Pitch(90))?')
-    custom_turtle5 : bpy.props.StringProperty(name="custom turtle 5", default='axiom("F X")?')
-    custom_turtle6 : bpy.props.StringProperty(name="custom turtle 6", default='rule("X","X + Y F +")?')
-    custom_turtle7 : bpy.props.StringProperty(name="custom turtle 7", default='rule("Y","- F X - Y")?;')
-    custom_turtle8 : bpy.props.StringProperty(name="custom turtle 8", default='round()')
-    custom_turtle9 : bpy.props.StringProperty(name="custom turtle 9", default='')
+    custom_turtle0: bpy.props.StringProperty(name="custom turtle 0", default='token("X", Turtle::Nop)?')
+    custom_turtle1: bpy.props.StringProperty(name="custom turtle 1", default='token("Y", Turtle::Nop)?')
+    custom_turtle2: bpy.props.StringProperty(name="custom turtle 2", default='token("F", Turtle::Forward(1))?')
+    custom_turtle3: bpy.props.StringProperty(name="custom turtle 3", default='token("+", Turtle::Yaw(-90))?')
+    custom_turtle4: bpy.props.StringProperty(name="custom turtle 4", default='token("-", Turtle::Pitch(90))?')
+    custom_turtle5: bpy.props.StringProperty(name="custom turtle 5", default='axiom("F X")?')
+    custom_turtle6: bpy.props.StringProperty(name="custom turtle 6", default='rule("X","X + Y F +")?')
+    custom_turtle7: bpy.props.StringProperty(name="custom turtle 7", default='rule("Y","- F X - Y")?;')
+    custom_turtle8: bpy.props.StringProperty(name="custom turtle 8", default='round()')
+    custom_turtle9: bpy.props.StringProperty(name="custom turtle 9", default='')
 
     def invoke(self, context, event):
         # load custom settings
-        #settings_load(self)
+        # settings_load(self)
         return self.execute(context)
 
     def execute(self, context):
 
         check_toxicblend_version()
-        #settings_write(self)
+        # settings_write(self)
         cursor_location = bpy.context.scene.cursor.location.copy()
         channel_opt = [('grpc.max_send_message_length', 512 * 1024 * 1024),
                        ('grpc.max_receive_message_length', 512 * 1024 * 1024)]
@@ -294,18 +295,17 @@ class TbAddLindenmayerSystems(bpy.types.Operator):
                 if self.cmd_variant == "CUSTOM_TURTLE":
                     opt = command.options.add()
                     opt.key = "CUSTOM_TURTLE"
-                    opt.value = str(self.custom_turtle0) + "\n" +\
-                                str(self.custom_turtle1) + "\n" +\
-                                str(self.custom_turtle2) + "\n" +\
-                                str(self.custom_turtle3) + "\n" +\
-                                str(self.custom_turtle4) + "\n" +\
-                                str(self.custom_turtle5) + "\n" +\
-                                str(self.custom_turtle6) + "\n" +\
-                                str(self.custom_turtle7) + "\n" +\
-                                str(self.custom_turtle8) + "\n" +\
+                    opt.value = str(self.custom_turtle0) + "\n" + \
+                                str(self.custom_turtle1) + "\n" + \
+                                str(self.custom_turtle2) + "\n" + \
+                                str(self.custom_turtle3) + "\n" + \
+                                str(self.custom_turtle4) + "\n" + \
+                                str(self.custom_turtle5) + "\n" + \
+                                str(self.custom_turtle6) + "\n" + \
+                                str(self.custom_turtle7) + "\n" + \
+                                str(self.custom_turtle8) + "\n" + \
                                 str(self.custom_turtle9)
 
-
                 pb_response = stub.execute(command)
                 handle_response(pb_response)
 
@@ -313,13 +313,14 @@ class TbAddLindenmayerSystems(bpy.types.Operator):
                     raise ToxicblendException("No return models found")
                 else:
                     mesh = bpy.data.meshes.new("lsystems")
-                    #bm = bmesh.new()
-                    #bm.verts.ensure_lookup_table()
+                    # bm = bmesh.new()
+                    # bm.verts.ensure_lookup_table()
 
-                    print("client received options: ", len(pb_response.options), " models64:", len(pb_response.models), " models32:", len(pb_response.models32))
+                    print("client received options: ", len(pb_response.options), " models64:", len(pb_response.models),
+                          " models32:", len(pb_response.models32))
                     handle_received_object(mesh, pb_response)
 
-                    #bm.to_mesh(mesh)
+                    # bm.to_mesh(mesh)
                     mesh.update()
 
                     # add the mesh as an object into the scene with this utility module
@@ -337,18 +338,19 @@ class TbAddLindenmayerSystems(bpy.types.Operator):
             self.report({'ERROR'}, str(e))
             return {'CANCELLED'}
 
+
 class TbAddVoxelSdf(bpy.types.Operator):
-    """Adds Lindenmayer systems edges/curves from local toxicblend server"""
+    """Adds sdf voxels from local toxicblend server"""
     bl_idname = "mesh.toxicblend_add_voxel_sdf"
     bl_label = "Toxicblend:Add Voxel Sdf systems"
     bl_options = {'REGISTER', 'UNDO'}  # enable undo for the operator.
 
-    location : bpy.props.FloatVectorProperty(
+    location: bpy.props.FloatVectorProperty(
         name="Location",
         subtype='TRANSLATION',
     )
 
-    rotation : bpy.props.FloatVectorProperty(
+    rotation: bpy.props.FloatVectorProperty(
         name="Rotation",
         subtype='EULER',
     )
@@ -360,7 +362,7 @@ class TbAddVoxelSdf(bpy.types.Operator):
         ('CURSOR', "3D Cursor", "Use the 3D cursor orientation for the new object")
     )
 
-    align : bpy.props.EnumProperty(
+    align: bpy.props.EnumProperty(
         name="Align",
         items=align_items,
         default='WORLD',
@@ -368,61 +370,61 @@ class TbAddVoxelSdf(bpy.types.Operator):
     )
 
     t_param: FloatProperty(
-            name="t parameter",
-            description="Thickness: abs(dot(sin(point), cos(point.zxy)) - b) - t",
-            default=1.0,
-            min=0.0001,
-            max=4.9999,
-            precision=6,
-            subtype='FACTOR'
+        name="t parameter",
+        description="Thickness: abs(dot(sin(point), cos(point.zxy)) - b) - t",
+        default=1.0,
+        min=0.0001,
+        max=4.9999,
+        precision=6,
+        subtype='FACTOR'
     )
 
     s_param: FloatProperty(
-            name="s parameter",
-            description="Scale",
-            default=1.0,
-            min=0.0001,
-            max=4.9999,
-            precision=6,
-            subtype='FACTOR'
+        name="s parameter",
+        description="Scale",
+        default=1.0,
+        min=0.0001,
+        max=4.9999,
+        precision=6,
+        subtype='FACTOR'
     )
 
     b_param: FloatProperty(
-            name="b parameter",
-            description="Tube thickness: abs(dot(sin(point), cos(point.zxy)) - b) - t",
-            default=1.0,
-            min=-5.000,
-            max=5.000,
-            precision=6,
-            subtype='FACTOR'
+        name="b parameter",
+        description="Tube thickness: abs(dot(sin(point), cos(point.zxy)) - b) - t",
+        default=1.0,
+        min=-5.000,
+        max=5.000,
+        precision=6,
+        subtype='FACTOR'
     )
     x_param: FloatProperty(
-             name="x parameter",
-             description="cos(x) and sin(x) multiplier",
-             default=1.0,
-             min=-5.000,
-             max=5.000,
-             precision=6,
-             subtype='FACTOR'
+        name="x parameter",
+        description="cos(x) and sin(x) multiplier",
+        default=1.0,
+        min=-5.000,
+        max=5.000,
+        precision=6,
+        subtype='FACTOR'
     )
     y_param: FloatProperty(
-             name="y parameter",
-             description="cos(y) and sin(y) multiplier",
-             default=1.0,
-             min=-5.000,
-             max=5.000,
-             precision=6,
-             subtype='FACTOR'
-     )
+        name="y parameter",
+        description="cos(y) and sin(y) multiplier",
+        default=1.0,
+        min=-5.000,
+        max=5.000,
+        precision=6,
+        subtype='FACTOR'
+    )
     z_param: FloatProperty(
-             name="z parameter",
-             description="cos(z) and sin(z) multiplier",
-             default=1.0,
-             min=-5.000,
-             max=5.000,
-             precision=6,
-             subtype='FACTOR'
-     )
+        name="z parameter",
+        description="cos(z) and sin(z) multiplier",
+        default=1.0,
+        min=-5.000,
+        max=5.000,
+        precision=6,
+        subtype='FACTOR'
+    )
     divisions: FloatProperty(
         name="Voxel Divisions",
         description="The longest axis of the model will be divided up into this number of voxels, the other axes will have proportionally number of voxels",
@@ -432,28 +434,34 @@ class TbAddVoxelSdf(bpy.types.Operator):
         precision=1,
         subtype='FACTOR'
     )
-    #plug_ends: BoolProperty(
+    # plug_ends: BoolProperty(
     #    name="Plug ends",
     #    description="Wall off outer edges with mesh",
     #    default=True
-    #)
+    # )
+
+    backend_variant_items = (#("_BB", "Building blocks", "use building blocks backend"),
+                             #("_SAFT", "Saft", "use saft backend"),
+                             ("_FSN", "FSN", "use fast_surface_nets backend"),
+                             )
+    cmd_backend: bpy.props.EnumProperty(name="Backend", items=backend_variant_items, default="_FSN")
 
     def invoke(self, context, event):
         # load custom settings
-        #settings_load(self)
+        # settings_load(self)
         return self.execute(context)
 
     def execute(self, context):
 
         check_toxicblend_version()
-        #settings_write(self)
+        # settings_write(self)
         cursor_location = bpy.context.scene.cursor.location.copy()
         channel_opt = [('grpc.max_send_message_length', 512 * 1024 * 1024),
                        ('grpc.max_receive_message_length', 512 * 1024 * 1024)]
         try:
             with grpc.insecure_channel(SERVER_URL, options=channel_opt) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
-                command = toxicblend_pb2.Command(command='sdf')
+                command = toxicblend_pb2.Command(command='sdf' + str(self.cmd_backend).lower())
 
                 opt = command.options.add()
                 opt.key = "T"
@@ -483,9 +491,9 @@ class TbAddVoxelSdf(bpy.types.Operator):
                 opt.key = "DIVISIONS"
                 opt.value = str(self.divisions)
 
-                #opt = command.options.add()
-                #opt.key = "PLUG_ENDS"
-                #opt.value = str(self.plug_ends)
+                # opt = command.options.add()
+                # opt.key = "PLUG_ENDS"
+                # opt.value = str(self.plug_ends)
 
                 pb_response = stub.execute(command)
                 handle_response(pb_response)
@@ -494,13 +502,14 @@ class TbAddVoxelSdf(bpy.types.Operator):
                     raise ToxicblendException("No return models found")
                 else:
                     mesh = bpy.data.meshes.new("sdf")
-                    #bm = bmesh.new()
-                    #bm.verts.ensure_lookup_table()
+                    # bm = bmesh.new()
+                    # bm.verts.ensure_lookup_table()
 
-                    print("client received options: ", len(pb_response.options), " models64:", len(pb_response.models), " models32:", len(pb_response.models32))
+                    print("client received options: ", len(pb_response.options), " models64:", len(pb_response.models),
+                          " models32:", len(pb_response.models32))
                     handle_received_object(mesh, pb_response)
 
-                    #bm.to_mesh(mesh)
+                    # bm.to_mesh(mesh)
                     mesh.update()
 
                     # add the mesh as an object into the scene with this utility module
