@@ -12,20 +12,29 @@
     unused_imports,
     unused_variables
 )]
-
 #![cfg_attr(feature = "hash_drain_filter", feature(hash_drain_filter))]
 #![cfg_attr(feature = "map_first_last", feature(map_first_last))]
 
+#[cfg(feature = "saft")]
+extern crate saft_cr as saft;
 mod cmd_2d_outline;
+// Replacing building_blocks with fast_surface_nets, but keeping the files for now
+//mod cmd_bb_sdf;
+//mod cmd_bb_voxel;
 mod cmd_centerline;
+mod cmd_fsn_sdf;
+mod cmd_fsn_voxel;
 mod cmd_knife_intersect;
 mod cmd_lsystems;
-mod cmd_sdf;
+#[cfg(feature = "saft")]
+mod cmd_saft_voxel;
+//#[cfg(feature = "saft")]
+//mod cmd_saft_sdf;
 mod cmd_simplify;
 mod cmd_voronoi;
 mod cmd_voronoi_mesh;
-mod cmd_voxel;
 mod lsystems_3d;
+mod type_utils;
 mod voronoi_utils;
 
 use crate::toxicblend_pb::Command as PB_Command;
@@ -75,6 +84,32 @@ impl PB_Vertex {
     }
 }
 
+pub(crate) trait GrowingVob {
+    fn fill(initial_size: usize) -> vob::Vob<u32>;
+    fn set_grow(&mut self, bit: usize, state: bool) -> bool;
+    /// get with default value: false
+    fn get_f(&self, bit: usize) -> bool;
+}
+
+impl GrowingVob for vob::Vob<u32> {
+    fn fill(initial_size: usize) -> Self {
+        let mut v: vob::Vob<u32> = vob::Vob::<u32>::new_with_storage_type(0);
+        v.resize(initial_size, false);
+        v
+    }
+    #[inline]
+    fn set_grow(&mut self, bit: usize, state: bool) -> bool {
+        if bit >= self.len() {
+            self.resize(bit + 512, false);
+        }
+        self.set(bit, state)
+    }
+    #[inline]
+    fn get_f(&self, bit: usize) -> bool {
+        self.get(bit).unwrap_or(false)
+    }
+}
+
 /// converts a Point2 to Point3 using the XY coordinates, sets Z to zero.
 #[inline(always)]
 pub fn xy_to_3d(point: &cgmath::Point2<f64>) -> cgmath::Point3<f64> {
@@ -95,9 +130,18 @@ fn xy_to_2d(point: &cgmath::Point3<f64>) -> cgmath::Point2<f64> {
 
 #[derive(thiserror::Error, Debug)]
 pub enum TBError {
+    #[error("In compatibleVersion : {0}")]
+    InCompatibleVersion(String),
+
+    #[error("Not implemented : {0}")]
+    NotImplemented(String),
+
     #[error("Overflow error: {0}")]
     Overflow(String),
 
+    #[error("Feature not active: {0}")]
+    DisabledFeature(String),
+
     #[error("Unknown command received: {0}")]
     UnknownCommand(String),
 
@@ -139,6 +183,10 @@ pub enum TBError {
 
     #[error("Error in LSystems {0}")]
     LSystems3D(String),
+
+    #[cfg(feature = "saft")]
+    #[error(transparent)]
+    SaftError(#[from] saft::Error),
 }
 
 #[derive(Debug, Default)]
@@ -170,9 +218,26 @@ impl ToxicBlendService for TheToxicBlendService {
             "centerline" => cmd_centerline::command(a_command, options_map),
             "voronoi_mesh" => cmd_voronoi_mesh::command(a_command, options_map),
             "voronoi" => cmd_voronoi::command(a_command, options_map),
-            "voxel" => cmd_voxel::command(a_command, options_map),
+            "voxel" => Err(TBError::InCompatibleVersion(String::from(
+                "Please update your toxicblend blender addons",
+            ))),
+            //"voxel_bb" => cmd_bb_voxel::command(a_command, options_map),
+            "voxel_fsn" => cmd_fsn_voxel::command(a_command, options_map),
+            #[cfg(feature = "saft")]
+            "voxel_saft" => cmd_saft_voxel::command(a_command, options_map),
+            #[cfg(not(feature = "saft"))]
+            "voxel_saft" => Err(TBError::DisabledFeature(String::from(
+                "The feature 'saft' is not enabled in the server",
+            ))),
             "lsystems" => cmd_lsystems::command(a_command, options_map),
-            "sdf" => cmd_sdf::command(a_command, options_map),
+            "sdf" => Err(TBError::InCompatibleVersion(String::from(
+                "Please update your toxicblend blender addons",
+            ))),
+            //"sdf_bb" => cmd_bb_sdf::command(a_command, options_map),
+            "sdf_fsn" => cmd_fsn_sdf::command(a_command, options_map),
+            "sdf_saft" => Err(TBError::NotImplemented(String::from(
+                "The sdf backend 'saft' is not implemented (yet)",
+            ))),
             _ => Err(TBError::UnknownCommand(a_command.command.clone())),
         };
         // convert TBError to a valid Error message
